//index.js
import ParseConfig from '../../utils/ParseConfig'
import MinaTouch from 'mina-touch';
import VEditor from '../../utils/veditor.js';
import Affine from '../../utils/affine.js';
import {
  uuid
} from '../../utils/cropCanvas.js';
import Upload from '../../utils/Upload.js';
import ReplaceableJson from "../../utils/ReplaceableJson.js";

let app = getApp();

Page({
  data: {
    groups: [],
    orderList: [],
    currentGroup: {},
    currentIndex: 0,
    currentFontIndex: 0,
    maxLength: 0,
    currentData: 0,
    startWidth: 0,
    startHeight: 0,
    isOpen: false,
    isTrue: false, //是否更换素材
    page: 'color',
    allColor: ['#ffffff', '#989898', '#323232', '#333333', '#000000', '#FD7E7A', '#E7403A', '#F091B1', '#BB67C9', '#7B25A1', '#9572D6', '#6136B1', '#A3A6DA', '#373C9E', '#8BD1FA', '#2F78BA', '#2D79B4', '#54C7DC', '#32828F', '#89CCC3', '#87C582', '#4A8D40', '#498E3E', '#E3ED9F', '#F6E959', '#FBCA82', '#F58C30', '#FCAA94', '#E05129', '#A2877E', '#574235', '#573535'],
    fontColor: "",
    p: 0,
    value: 0, //滑块重置为0
    r: "",
    colorIndex: -1,
    strokeColorIndex: -1,
    inputText: "", //输入的值
    valueText: "", //输入框默认值
    angle: 0,
    canvasImgWidth: 20,
    canvasImgHeight: 320,
    videoShow: "picShow",
    videoSrc: "",
    videoPageX: 0,
    videoWidth: 0,
    videoDuration: 0,
    currentTime: 0,
    mutedTrue: false,
    pickerHeight: 0,
  },
  pageX: 0,
  pageY: 0,
  duration: 15,
  unit: 10,
  videoPageX: 0,
  startTime: 0,
  endTime: 0,
  ...VEditor,
  moveTimer: null,
  moveTrue: false,
  force: 0,
  rotateImg: 0,
  move: false, //是否两个手指触摸

  onLoad(options) {
    let _this = this
    wx.request({
      url: options.item,
      method: "get",
      header: {
        'content-type': 'application/json',
      },
      success: function (res) {
        let item = res.data
        let json = new ParseConfig(item);
        let allMaterial = [];
        for (let key in json.groups) {
          let obj = {};
          if (app.all.bgList.length == 0) {
            obj.bg = "";
          } else {
            obj.bg = app.all.bgList[json.groups[key].backgroundImage];
          }

          if (app.all.fgList.length == 0) {
            obj.fg = "";
          } else {
            obj.fg = app.all.fgList[json.groups[key].prospectImage];
          }

          obj.all = json.groups[key].allMaterial;
          obj.groupKey = key;
          allMaterial.push(obj);
        }
        app.all.allMaterial = allMaterial;
        app.all.parseConfig = json;
        _this.setData({
          groups: allMaterial,
          orderList: json.orderList,
          // canvasImgWidth: width,
        })
        _this.initEditor(json, allMaterial)

        new MinaTouch(_this, 'touch1', {
          touchStart: function (e) {
            _this.force = e.touches[0].force;
            _this.moveTrue = true;
            if (e.target.dataset.item && e.touches.length !== 0) {
              _this.setData({
                currentIndex: e.target.dataset.index,
              })
              _this.pageX = e.touches[0].pageX;
              _this.pageY = e.touches[0].pageY;
            }
          },
          touchMove: function (e) {
            const {
              currentIndex,
              currentGroup,
            } = _this.data
            if (e.target.dataset.index == currentIndex && e.type == "touchmove") {
              let {
                newMatrix,
                scale
              } = currentGroup.picArray[currentIndex];
              let {
                scaleBox
              } = currentGroup
              let moveX = (e.touches[0].pageX - _this.pageX) / scaleBox / scale[0];
              let moveY = (e.touches[0].pageY - _this.pageY) / scaleBox / scale[1];
              _this.pageX = e.touches[0].pageX
              _this.pageY = e.touches[0].pageY
              let options = {
                p: {
                  x: moveX,
                  y: moveY,
                }
              }
              newMatrix.multiply(_this.matrix(options));
              _this.asyncSetData(newMatrix)
            }
          },
          touchEnd: function (e) {
            const {
              currentGroup,
              currentIndex,
              groups,
              angle,
              currentData,
            } = _this.data
            if (currentGroup.picArray.length !== 0) {
              currentGroup.picArray[currentIndex].pageX = _this.pageX;
              currentGroup.picArray[currentIndex].pageY = _this.pageY;
              currentGroup.picArray[currentIndex].rotateImg = _this.rotateImg;
              groups[currentData] = currentGroup
              _this.setData({
                groups: groups
              })
            }
          },
          rotate: function (e) {
            const {
              currentIndex,
              currentGroup
            } = _this.data
            let {
              newMatrix,
            } = currentGroup.picArray[currentIndex];

            // evt.angle代表两个手指旋转的角度
            if (e.target.dataset.index == currentIndex && e.type == "rotate" && _this.move) {
              let options = {
                r: e.angle,
              }
              _this.rotateImg = e.angle + _this.rotateImg;
              _this.setData({
                angle: e.angle
              })
              newMatrix.multiply(_this.matrix(options));
              _this.asyncSetData(newMatrix)
            }
          },
          pinch: function (e) {
            //e.zoom代表两个手指缩放的比例(多次缩放的累计值),e.singleZoom代表单次回调中两个手指缩放的比例
            const {
              currentIndex,
              currentGroup
            } = _this.data
            let {
              imgSize
            } = currentGroup.picArray[currentIndex];
            if (e.target.dataset.index == currentIndex && e.type == "pinch" && _this.move) {
              let options = {
                scale: {
                  scaleImg: e.zoom,
                },
              }
              _this.asyncSetData(options)
            }
          },
          multipointStart: function () {
            _this.move = true;
          }, //一个手指以上触摸屏幕触发
          multipointEnd: function () {
            _this.move = false;
          }, //当手指离开，屏幕只剩一个手指或零个手指触发
        })
      },
    })
  },
  //批量导入
  importsTap(e) {
    let {
      currentGroup,
      groups,
      currentData
    } = this.data
    let _this = this;
    let picsMatrix = currentGroup.picArray;
    let anchor = {
        x: 0,
        y: 0
      },
      position = {
        x: 0,
        y: 0
      },
      zoom = {
        x: 1,
        y: 1
      },
      rotation = 0;
    if (e.target.id == "up") {
      if (currentGroup.picArray.length !== 0) {
        wx.chooseImage({
          count: currentGroup.picArray.length,
          sourceType: ["album"],
          success(res) {
            const currentFile = res.tempFiles;
            for (let r in currentFile) {
              for (let i in currentGroup.picArray) {
                if (i == r) {
                  currentGroup.picArray[i].assetName = currentFile[r].path;
                  currentGroup.picArray[i].isVideo = false;
                }
              }
            }
            groups[currentData] = currentGroup;
            _this.setData({
              currentGroup,
              groups,
              isTrue: true
            });
          }
        });
      }
    } else if (e.target.id == "reset") {
      for (let index in picsMatrix) {
        let moveMatrix = new Affine();
        moveMatrix.set(anchor, position, zoom, rotation);
        picsMatrix[index].newMatrix = moveMatrix;
        picsMatrix[index].scaleImg = 1;
      };
      groups[currentData] = currentGroup;
      _this.setData({
        currentGroup,
        groups
      })
    }
  },
  // 编辑组的切换
  bindGroup(e) {
    const {
      num
    } = e.currentTarget.dataset
    const {
      groups
    } = this.data
    this.rotateImg = 0;
    this.setData({
      currentData: num,
      currentGroup: groups[num],
      canvasImgWidth: groups[num].widthBox,
    })
  },
  // 矩阵方法
  matrix(options) {
    let moveMatrix = new Affine(),
      anchor = {
        x: 0,
        y: 0
      },
      position = {
        x: 0,
        y: 0
      },
      zoom = {
        x: 1,
        y: 1
      },
      rotation = 0;


    if (options.a) {
      anchor = options.a;
    }

    if (options.p) {
      position = options.p;
    }

    if (options.s) {
      zoom = options.s;
    }

    if (options.r) {
      rotation = options.r;
    }
    moveMatrix.set(anchor, position, zoom, rotation);
    return moveMatrix
  },
  // 页面初始化
  initEditor: function (configJson, all) {
    VEditor.initEditor(configJson, {}, this, all);
  },
  //延迟函数
  asyncSetData: function (options) {
    this.throttle(options).then(res => {

    });
  },
  throttle: function (newData) {
    // 延迟函数变量
    let delayNewState = null;
    let newState = {
      ...delayNewState,
      // ...newData,
    };
    let {
      currentGroup,
      currentIndex
    } = this.data
    if (newData.scale) {
      currentGroup.picArray[currentIndex].scaleImg = newData.scale.scaleImg
    } else {
      currentGroup.picArray[currentIndex].newMatrix = newData
    }
    return new Promise(resolve => {
      clearTimeout(this.moveTimer);
      this.moveTimer = setTimeout(() => {
        this.moveTrue = false;
        this.setData({
          currentGroup,
        });
        resolve(newState);
        newState = null;
      }, 5);
    })
  },
  text() {
    let {
      page
    } = this.data
    if (page === "color") {
      this.setData({
        page: "text",
        isOpen: true
      })
    }
  },
  color() {
    this.setData({
      page: "color",
      pickerHeight: 235
    })
  },
  // 字体颜色
  fontColorChange(e) {
    const {
      selected,
      index
    } = e.currentTarget.dataset;
    let {
      currentGroup,
      currentFontIndex,
    } = this.data
    let text = currentGroup.textareaArray[currentFontIndex];
    text.fontSetting.textColor = selected;
    this.setData({
      currentGroup,
      colorIndex: index,
      isTrue: true
    })
  },
  strokeColorChange(e) {
    const {
      selected,
      index
    } = e.currentTarget.dataset
    let {
      currentGroup,
      currentFontIndex,
    } = this.data
    let text = currentGroup.textareaArray[currentFontIndex];
    text.fontSetting.strokeColor = selected;
    this.setData({
      currentGroup,
      strokeColorIndex: index,
      isTrue: true
    })
  },
  // 描边滑块取值
  sliderchangeing(e) {
    let {
      value
    } = e.detail
    let {
      currentGroup,
      currentFontIndex,
    } = this.data
    let text = currentGroup.textareaArray[currentFontIndex];
    text.fontSetting.strokeWidth = value;
    this.setData({
      currentGroup,
    })
  },
  inputChange(e) {
    let {
      currentGroup,
      currentFontIndex,
    } = this.data
    let text = currentGroup.textareaArray[currentFontIndex];
    text.fontSetting.defaultText = e.detail.value;
    this.setData({
      inputText: e.detail.value,
      valueText: e.detail.value,
      currentGroup,
      isTrue: true
    })
  },
  inputFocus(e) {
    this.setData({
      page: "text",
      inputText: e.detail.value,
      valueText: e.detail.value,
      pickerHeight: e.detail.height + 75
    })
  },
  inputBlur(e) {
    let {
      page
    } = this.data
    if (page !== "color") {
      this.setData({
        pickerHeight: 90
      })
    }
  },
  // 取消
  tapHideModal() {
    this.setData({
      isOpen: false,
      fontColor: "",
      p: 0,
      r: "",
      value: 0
    })
  },
  closeModal() {
    this.setData({
      isOpen: false,
    })
  },

  async uploadCropFile() {
    app.loading();
    const _this = this;
    let {
      groups,
    } = this.data
    let upload = new Upload({
        getTokenUrl: app.hostUrl + "/TemplateDemo.php?act=get_upload_key",
        token: wx.getStorageSync('token').token,
        sourceHost: app.sourceHost,
      }),
      replaceableJson = {};
    for (let i in groups) {
      let picCanvas = groups[i].picArray,
        textArray = groups[i].textareaArray
      replaceableJson[groups[i]["groupKey"]] = [];
      for (let j in picCanvas) {
        const currentCanvas = picCanvas[j],
          imageData = {},
          src = currentCanvas.assetName;
        imageData["type"] = currentCanvas["type"];
        imageData["key"] = currentCanvas["key"];
        imageData["isVideo"] = currentCanvas["isVideo"];

        if (imageData["isVideo"]) {
          let s = currentCanvas["newMatrix"]["mA"],
            r = currentCanvas["rotateImg"],
            x = currentCanvas["newMatrix"]["mE"],
            y = currentCanvas["newMatrix"]["mF"];
          imageData["videoTransform"] = this.matrix({
            a: {
              x: currentCanvas["videoWidth"] / 2,
              y: currentCanvas["videoHeight"] / 2,
            },
            p: {
              x: x,
              y: y,
            },
            s: {
              x: s,
              y: s,
            },
            r: r,
          });
          imageData["adapt_type"] = 4;
          imageData["videoClip"] = currentCanvas["videoClip"];
          let point = src.lastIndexOf("."),
            type = src.substr(point);
          const fullUrl = await upload.uploadFile(src, `${uuid()}${type}`);
          imageData["imgUrl"] = fullUrl;
        } else {
          let editSize = [currentCanvas.originSize[0], currentCanvas.originSize[1]],
            editScale = [currentCanvas.scale[0], currentCanvas.scale[1]],
            id = "myCanvas",
            transform = {
              p: {
                x: currentCanvas["newMatrix"]["mE"],
                y: currentCanvas["newMatrix"]["mF"]
              },
              s: {
                x: currentCanvas.scaleImg,
                y: currentCanvas.scaleImg,
              },
              r: currentCanvas.rotateImg
            };
          const {
            path,
            fileName
          } = await _this.cropImage(src, editSize, transform, _this, editScale);
          const fullUrl = await upload.uploadFile(path, fileName);
          imageData["imgUrl"] = fullUrl;
        }
        replaceableJson[groups[i]["groupKey"]].push(imageData);
      }

      for (let t in textArray) {
        const currentText = textArray[t],
          textData = {};
        textData["type"] = currentText["type"];
        textData["key"] = currentText["key"];
        textData["fontSetting"] = currentText["fontSetting"];
        replaceableJson[groups[i]["groupKey"]].push(textData);
      }
    }

    return replaceableJson;
  },
  // 立即生成
  generateTap() {
    let _this = this
    if (_this.data.isTrue) {
      _this.uploadCropFile().then(res => {
        let id = app.all.id;
        let {
          orderList
        } = _this.data
        let json = new ReplaceableJson(
          res,
          orderList,
          id
        ).getJson();

        wx.request({
          url: app.hostUrl + '/TemplateDemo.php?act=start_render',
          method: 'POST',
          header: {
            'content-type': 'application/json',
            'login-token': wx.getStorageSync('token').token,
          },
          data: {
            ...json
          },
          success: (res => {
            _this.socketInterFace(res.data.data.res.data.orderId,res.data.data.record_id,res.data.data.video_url)
          }),
          fail: (res => {
            wx.hideLoading();
            app.showError("渲染失败");
          })
        })
      });
    } else {
      app.showError("请选择替换素材");
    }

  },

  /**
   * 轮询接口
   * */
  socketInterFace: function (value,recordId,url) {
    let _this = this;
    wx.request({
      url: app.hostUrl + '/TemplateDemo.php?act=get_status&record_id=' + recordId,
      method: 'GET',
      data: {
        order_id: value,
      },
      header: {
        'content-type': 'application/json',
      },
      success: function (res) {
        const {
          data
        } = res.data;
       
        if (data.status === "0" ||  data.status === "2") {
          setTimeout(function () {
            _this.socketInterFace(value,recordId,url);
          }, 1000)
        } else if (data.status === "1" && data.result_video_url !== '') {
          wx.hideLoading();
          wx.navigateTo({
            url: '../video/video?item=' + url,
          })
        } else {
          wx.hideLoading();
          app.showError("渲染失败");
        }
      }
    })
  },

  // 截图
  cropImage(src, editSize, transform, __this__, editScale) {
    let canvasInfo = {},
      _this = this;
    return new Promise((resolve, reject) => {
      canvasInfo["context"] = wx.createCanvasContext("cropImageCanvas", __this__);
      const context = canvasInfo["context"],
        editWidth = editSize[0],
        editHeight = editSize[1];
      _this.setData({
        canvasImgWidth: editWidth,
        canvasImgHeight: editHeight,
      }, () => {
        wx.getImageInfo({
          src: src,
          success: (res) => {
            const imageWidth = res.width,
              imageHeight = res.height;
            src = res.path;
            context.save();
            context.clearRect(0, 0, editWidth, editHeight);
            let matrix = new Affine(),
              a = {
                x: imageWidth / 2,
                y: imageHeight / 2,
              },
              p = {
                x: imageWidth / 2 + transform["p"]["x"],
                y: imageHeight / 2 + transform["p"]["y"],
              },
              s = {
                x: transform["s"]["x"],
                y: transform["s"]["y"],
              },
              r = transform["r"];
            matrix.set(a, p, s, r);
            context.setTransform(matrix.mA, matrix.mB, matrix.mC, matrix.mD, matrix.mE, matrix.mF);
            context.drawImage(src, 0, 0, imageWidth, imageHeight);
            context.restore();
            context.draw(true, () => {
              wx.canvasToTempFilePath({
                x: 0,
                y: 0,
                width: editWidth,
                height: editHeight,
                fileType: "png",
                destWidth: editWidth,
                destHeight: editHeight,
                canvasId: "cropImageCanvas",
                success(res) {
                  resolve({
                    path: res.tempFilePath,
                    fileName: `${uuid()}.png`,
                  });
                },
                fail(error) {
                  app.showError("渲染失败");
                },
              });
            });
          },
        });
      });
    }).catch(err => {
      app.showError("渲染失败");
    });
  },

  moveChange(e) {
    const {
      currentGroup,
      currentIndex
    } = this.data
    let fps = app.all.parseConfig.fps;
    let duration = currentGroup.picArray[currentIndex].asset.ui.duration / fps;

    let x = e.detail.x;
    let timer = 375 / this.duration;
    let currentTime = x / timer;
    let endTime = currentTime + duration;

    let VideoContext = wx.createVideoContext("videoClip", this)
    VideoContext.pause();

    setTimeout(() => {
      VideoContext.seek(currentTime)
      this.setData({
        currentTime: currentTime
      }, () => {
        VideoContext.play();
      })
    }, 1000);

    currentGroup.picArray[currentIndex].videoClip = {
      startTime: currentTime,
      endTime: endTime
    };
  },
  next() {
    this.setData({
      videoShow: "picShow",
    })
  },
  mutedTap() {
    this.setData({
      mutedTrue: true,
    })
  },
  touchTap(e) {
    let _this = this
    if (_this.force == e.touches[0].force && _this.moveTrue == true) {
      let {
        currentGroup,
        groups,
        currentIndex,
        currentData,
      } = _this.data
      if (e.target.id) {
        _this.setData({
          maxLength: e.target.dataset.max,
          currentFontIndex: e.target.dataset.i,
        })
        let text = currentGroup.textareaArray[e.target.dataset.i].fontSetting;
        _this.setData({
          fontColor: text.textColor,
          p: text.strokeWidth,
          r: text.strokeColor,
          valueText: text.defaultText,
          isOpen: true,
          page: "text"
        })
      }
      if (e.target.dataset.item) {
        wx.chooseImage({
          count: 1,
          sourceType: ["album", "camera"],
          // maxDuration: 300,
          // camera: "back",
          success(res) {
            const currentFile = res.tempFiles[0];
            let tempPath = currentFile.path;
            for (let i in currentGroup.picArray) {
              if (e.target.dataset.item == currentGroup.picArray[i].key) {
                currentGroup.picArray[i].assetName = tempPath;
                currentGroup.picArray[i].isVideo = false;
                wx.getImageInfo({
                  src: tempPath,
                  success: (res) => {
                    currentGroup.picArray[i].imgSize[0] = res.width;
                    currentGroup.picArray[i].imgSize[1] = res.height;
                    groups[currentData] = currentGroup;
                    _this.setData({
                      currentGroup,
                      groups,
                      isTrue: true
                    });
                  }
                })
              }
            }
            // fileType = currentFile.fileType,
            // duration = Math.floor(currentFile.duration);
            // _this.duration = currentFile.duration;
            // switch (fileType) {
            //   case "video":
            //     for (let i in currentGroup.picArray) {
            //       if (e.target.dataset.item == currentGroup.picArray[i].key) {
            //         currentGroup.picArray[i].assetName = tempPath;
            //         currentGroup.picArray[i].videoWidth = currentFile.width;
            //         currentGroup.picArray[i].videoHeight = currentFile.height;
            //         currentGroup.picArray[i].isVideo = true;
            //         currentGroup.picArray[i].videoSize[0] = currentFile.width;
            //         currentGroup.picArray[i].videoSize[1] = currentFile.height;
            //         groups[currentData] = currentGroup;
            //         let width = Number(currentGroup.picArray[currentIndex].asset.ui.duration / app.all.parseConfig.fps);
            //         let timer = 375 / Number(_this.duration);
            //         _this.setData({
            //           currentGroup,
            //           groups,
            //           videoShow: "videoShow",
            //           videoSrc: tempPath,
            //           videoWidth: width * timer,
            //           videoDuration: duration,
            //           isTrue: true
            //         });
            //       }
            //     }
            //     break;

            // case "image":


            // break;
            // }
          }
        });
      }
    }
  },

})