class Upload {

  constructor(options) {
    this.init(options);
  }

  init(options) {
    this.initialization();
    const {
      getTokenUrl,
      token,
      sourceHost
    } = options;
    this.getTokenUrl = getTokenUrl;
    this.__token__ = token;
    this.sourceHost = sourceHost;
  }

  initialization() {
    this.getTokenUrl = "";
    this.__token__ = "";
    this.sourceHost = "";
  }

  getFileType(fileName) {
    let point = fileName.lastIndexOf(".");
    return fileName.substr(point);
  }

  getUploadPath(fileName, dir) {
    const userDir = dir || "";
    return userDir + (new Date()).getTime() + fileName;
  }

  getToken() {
    const {
      getTokenUrl,
      __token__
    } = this;
    return new Promise((resolve, reject) => {
      wx.request({
        url: getTokenUrl,
        success(res) {
          resolve(res.data);
        }
      })
    });
  }

  getFormData(policy, path) {
    const formData = {
      "OSSAccessKeyId": policy["accessid"],
      "policy": policy["policy"],
      "Signature": policy["signature"],
      "key": path,
      "success_action_status": 200
    }
    return formData;
  }

  wxUploadFile(url, filePath, formData) {
    return new Promise((resolve, reject) => {
      wx.uploadFile({
        url: `https://${url}`,
        filePath: filePath,
        name: "file",
        formData: formData,
        success: function (res) {
          resolve();
        },
        fail: function (res) {
          
        }
      });
    });
  };

  async uploadFile(tempPath, fileName) {
    let policy = await this.getToken(),
      path = this.getUploadPath(fileName, policy.dir),
      formData = this.getFormData(policy, path);

    await this.wxUploadFile(policy["host"], tempPath, formData);
    return `${this.sourceHost}${path}`;
  }
}

export default Upload;